#
# note_cgi.py - Note CGI methods.
# Copyright (C) 2005  shinGETsu Project.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: note_cgi.py,v 1.10 2005/04/27 12:17:47 fuktommy Exp $
#

import cgi
import re

import config
import gateway
from cache import *

__version__ = "$Revision: 1.10 $"


class CGI(gateway.CGI):

    """Class for /note.cgi."""

    appli_type = "note"

    def run(self):
        path = self.path_info()
        form = cgi.FieldStorage(environ=self.environ, fp=self.stdin)

        if form.getfirst("cmd", "") == "post" and \
           form.getfirst("file", "").startswith("note_") and \
           self.environ["REQUEST_METHOD"] == "POST":
            self.do_post(path, form)
            datfile = form.getfirst("file", "")
            title = self.str_encode(self.file_decode(datfile))
            self.print302(self.note_cgi + self.sep + title)
        elif (path == "") or (path.find("/") > 0):
            self.print404()
        elif form.getfirst("mode", "") == "edit":
            self.print_edit(path)
        elif form.getfirst("mode", "") == "history":
            self.print_history(path)
        else:
            self.print_note(path)

    def print_note(self, path):
        cache = Cache(self.file_encode("note", path))
        if not re.search(config.robot, self.environ["HTTP_USER_AGENT"]):
            self.get_cache(cache)
        self.header(path)
        (buf, conflict) = self.read_note(cache)
        if (buf is not None) and ("body" in buf):
            body = self.html_format(buf["body"])
        else:
            body = ""
        self.stdout.write("<p>\n    %s<br />\n</p>" % body)
        if conflict:
            self.stdout.write("<p><em>%s</em></p>" % self.message["conflict"])
        self.stdout.write(
            '<p>\n' +
            '  <a href="' + self.note_cgi + "/" + self.str_encode(path) +
            '?mode=edit">' + self.message["edit"] + '</a> |\n' +
            '  <a href="' + self.note_cgi + "/" + self.str_encode(path) +
            '?mode=history">' + self.message["history"] + '</a>\n' +
            '</p>')
        self.remove_file_form(cache)
        self.footer()

    def read_note(self, cache):
        """Read note history."""
        buf = None
        ref = {}
        for r in cache:
            rec = cache[r]
            rec.load()
            if "remove_stamp" in rec:
                continue
            buf = rec
            if "base_stamp" in rec:
                ref[rec.idstr] = 0
            if ("base_stamp" in rec) and ("base_id" in rec):
                base = rec["base_stamp"] + "_" + rec["base_id"]
                if base in ref:
                    ref[base] += 1
        orphan = 0
        for key in ref:
            if ref[key] == 0:
                orphan += 1
        return (buf, (orphan>1))

    def print_edit(self, path):
        """Print edit page."""
        cache = Cache(self.file_encode("note", path))
        if not re.search(config.robot, self.environ["HTTP_USER_AGENT"]):
            self.get_cache(cache)
        self.header(path, deny_robot=True)
        (buf, conflict) = self.read_note(cache)
        if (buf is not None) and ("body" in buf):
            body = self.html_format(buf["body"])
        else:
            body = ""
        self.stdout.write("<p>\n    %s<br />\n</p>" % body)
        self.print_edit_form(cache, buf)
        if conflict:
            self.stdout.write("<p><em>%s</em></p>" % self.message["conflict"])
        self.remove_file_form(cache)
        self.footer()

    def print_history(self, path):
        """Print note history."""
        cache = Cache(self.file_encode("note", path))
        if not re.search(config.robot, self.environ["HTTP_USER_AGENT"]):
            self.get_cache(cache)
        self.header(path, deny_robot=True)
        self.stdout.write(
            '<form method="get" action="' + self.admin_cgi + '">\n' +
            '<p>\n' +
            '  <input type="hidden" name="cmd" value="delete" />\n' +
            '  <input type="hidden" name="file"' +
            ' value="'+ cache.datfile + '" />\n' +
            '</p>\n' +
            '<dl>\n')

        recs = []
        ref = {}
        for r in cache:
            rec = cache[r]
            rec.load()
            recs.append(rec)
            ref[rec.idstr] = 0
            if ("base_stamp" in rec) and ("base_id" in rec):
                base_idstr = rec["base_stamp"] + "_" + rec["base_id"]
                if base_idstr not in ref:
                    ref[base_idstr] = 1
                else:
                    ref[base_idstr] += 1
            else:
                ref[rec.idstr] = 1
                continue

        if recs:
            ref[recs[-1].idstr] = 1

        for rec in recs:
            stamp = self.localtime(rec["stamp"])

            if "body" in rec:
                body = self.html_format(rec["body"])
            else:
                body = ""

            base = ""
            if "base_id" in rec:
                base = rec["base_id"][:8]
                base = cgi.escape(base, True)
                if base:
                    base = '(<a href="#r%s">&gt;&gt;%s</a>)' % (base, base)

            remove = ""
            if ("remove_id" in rec) and ("remove_stamp" in rec):
                id = rec["remove_id"][:8]
                remove = '[[%s: <a href="#r%s">%s</a>]]<br />' % \
                         (self.message["remove"], id, id)

            id = rec["id"][:8]
            if ref[rec.idstr] > 0:
                conflict = ""
            else:
                conflict = "    <em>%s</em>\n" % self.message["conflict"]

            self.stdout.write(
                '  <dt id="r' + id + '">\n' +
                '    <input type="radio" name="record" value="' +
                rec.idstr + '" tabindex="1" accesskey="s" />\n' +
                '    ' + id + "\n" +
                '    ' + stamp + "\n" +
                '    ' + base + "\n" +
                conflict +
                '  </dt>\n' +
                '  <dd>' + body + '<br />' + remove + "<br /></dd>\n")

        self.stdout.write(
            "</dl>\n" +
            "<p>\n" +
            '  <input type="submit" value="' + self.message["del_record"] +
            '" tabindex="2" accesskey="d" />\n' +
            "</p>\n" +
            "</form>\n")

	(buf, conflict) = self.read_note(cache)
	self.print_edit_form(cache, buf)
        self.remove_file_form(cache)
        self.footer()

    def print_edit_form(self, cache, rec):
        if (rec is not None) and ("body" in rec):
            body = re.sub(r"<br>", "\n", rec["body"])
            stamp = rec["stamp"]
            id = rec["id"]
        else:
            body = ""
            stamp = ""
            id = ""

        self.stdout.write(
            '<form method="post" action="' + self.note_cgi + '"><p>\n' +
            '  <input type="hidden" name="cmd" value="post" />\n' +
            '  <input type="hidden" name="file"' +
            ' value="' + cache.datfile + '" />\n' +
            '  <input type="submit" value="' + self.message["post"] + '"' +
            ' name="submit" tabindex="1" accesskey="w" />\n' +
            '  <input type="checkbox" name="dopost" value="dopost"' +
            ' checked="checked" tabindex="2" accesskey="s" />\n' +
            '  ' + self.message["send"] +
            ' <input type="checkbox" value="error" name="error"' + 
            ' checked="checked" tabindex="2" accesskey="e" />\n' +
            '  ' + self.message["error"] + "<br />\n" +
            '  <textarea rows="20" cols="75" name="body"' +
            ' tabindex="8" accesskey="c">' + body + "</textarea>\n" +
            '  <input type="hidden" name="base_stamp"' +
            ' value="' + stamp + '" />\n' +
            '  <input type="hidden" name="base_id"' +
            ' value="' + id + '" />\n' +
            '</p></form>\n')
